@@ -1,7 +1,10 @@
 import asyncio
+import inspect
 import logging
+import os
 import random
 import re
+import socket
 import string
 import sys
 import uuid
@@ -18,6 +21,7 @@ from sanic_testing.testing import PORT
 
 from sanic import Sanic
 from sanic.constants import HTTP_METHODS
+from sanic.logging.formatter import AutoFormatter
 from sanic.router import Router
 from sanic.touchup.service import TouchUp
 
@@ -30,6 +34,19 @@ if sys.platform in ["win32", "cygwin"]:
     collect_ignore = ["test_worker.py"]
 
 
+def get_port():
+    sock = socket.socket()
+    sock.bind(
+        ("", 0)
+    )  # Bind to 0 port, so os will pick available port for us.
+    return sock.getsockname()[1]
+
+
+@pytest.fixture(scope="function")
+def port():
+    yield get_port()
+
+
 async def _handler(request):
     """
     Dummy placeholder method used for route resolver when creating a new
@@ -58,7 +75,6 @@ CACHE: Dict[str, Any] = {}
 
 
 class RouteStringGenerator:
-
     ROUTE_COUNT_PER_DEPTH = 100
     HTTP_METHODS = HTTP_METHODS
     ROUTE_PARAM_TYPES = ["str", "int", "float", "alpha", "uuid"]
@@ -152,6 +168,10 @@ def app(request):
     for target, method_name in TouchUp._registry:
         setattr(target, method_name, CACHE[method_name])
     Sanic._app_registry.clear()
+    AutoFormatter.SETUP = False
+    AutoFormatter.LOG_EXTRA = False
+    os.environ.pop("SANIC_LOG_EXTRA", None)
+    os.environ.pop("SANIC_NO_COLOR", None)
 
 
 @pytest.fixture(scope="function")
@@ -213,12 +233,12 @@ def ext_instance():
 
 
 @pytest.fixture(autouse=True)  # type: ignore
-def sanic_ext(ext_instance):  # noqa
-    sanic_ext = MagicMock(__version__="1.2.3")
-    sanic_ext.Extend = MagicMock()
-    sanic_ext.Extend.return_value = ext_instance
-    sys.modules["sanic_ext"] = sanic_ext
-    yield sanic_ext
+def mock_sanic_ext(ext_instance):  # noqa
+    mock_sanic_ext = MagicMock(__version__="1.2.3")
+    mock_sanic_ext.Extend = MagicMock()
+    mock_sanic_ext.Extend.return_value = ext_instance
+    sys.modules["sanic_ext"] = mock_sanic_ext
+    yield mock_sanic_ext
     with suppress(KeyError):
         del sys.modules["sanic_ext"]
 
@@ -232,3 +252,12 @@ def urlopen():
     urlopen.read = Mock()
     with patch("sanic.cli.inspector_client.urlopen", urlopen):
         yield urlopen
+
+
+@pytest.fixture(scope="module")
+def static_file_directory():
+    """The static directory to serve"""
+    current_file = inspect.getfile(inspect.currentframe())
+    current_directory = os.path.dirname(os.path.abspath(current_file))
+    static_directory = os.path.join(current_directory, "static")
+    return static_directory
