@@ -2,19 +2,25 @@ import asyncio
 import logging
 
 from collections import deque, namedtuple
+from unittest.mock import call
 
 import pytest
 import uvicorn
 
+from httpx import Headers
+from pytest import MonkeyPatch
+
 from sanic import Sanic
 from sanic.application.state import Mode
-from sanic.asgi import ASGIApp, MockTransport
+from sanic.asgi import Lifespan, MockTransport
 from sanic.exceptions import BadRequest, Forbidden, ServiceUnavailable
 from sanic.request import Request
 from sanic.response import json, text
 from sanic.server.websockets.connection import WebSocketConnection
 from sanic.signals import RESERVED_NAMESPACES
 
+from .conftest import get_port
+
 
 try:
     from unittest.mock import AsyncMock
@@ -50,6 +56,9 @@ def transport(message_stack, receive, send):
 
 @pytest.fixture
 def protocol(transport):
+    loop = asyncio.new_event_loop()
+    asyncio.set_event_loop(loop)
+    transport.loop = loop
     return transport.get_protocol()
 
 
@@ -88,7 +97,9 @@ def test_listeners_triggered(caplog):
         def install_signal_handlers(self):
             pass
 
-    config = uvicorn.Config(app=app, loop="asyncio", limit_max_requests=0)
+    config = uvicorn.Config(
+        app=app, loop="asyncio", limit_max_requests=0, port=get_port()
+    )
     server = CustomServer(config=config)
 
     start_message = (
@@ -116,10 +127,6 @@ def test_listeners_triggered(caplog):
         stop_message,
     ) not in caplog.record_tuples
 
-    all_tasks = asyncio.all_tasks(asyncio.get_event_loop())
-    for task in all_tasks:
-        task.cancel()
-
     assert before_server_start
     assert after_server_start
     assert before_server_stop
@@ -190,7 +197,9 @@ def test_listeners_triggered_async(app, caplog):
         def install_signal_handlers(self):
             pass
 
-    config = uvicorn.Config(app=app, loop="asyncio", limit_max_requests=0)
+    config = uvicorn.Config(
+        app=app, loop="asyncio", limit_max_requests=0, port=get_port()
+    )
     server = CustomServer(config=config)
 
     start_message = (
@@ -218,10 +227,6 @@ def test_listeners_triggered_async(app, caplog):
         stop_message,
     ) not in caplog.record_tuples
 
-    all_tasks = asyncio.all_tasks(asyncio.get_event_loop())
-    for task in all_tasks:
-        task.cancel()
-
     assert before_server_start
     assert after_server_start
     assert before_server_stop
@@ -266,16 +271,14 @@ def test_non_default_uvloop_config_raises_warning(app):
         def install_signal_handlers(self):
             pass
 
-    config = uvicorn.Config(app=app, loop="asyncio", limit_max_requests=0)
+    config = uvicorn.Config(
+        app=app, loop="asyncio", limit_max_requests=0, port=get_port()
+    )
     server = CustomServer(config=config)
 
     with pytest.warns(UserWarning) as records:
         server.run()
 
-    all_tasks = asyncio.all_tasks(asyncio.get_event_loop())
-    for task in all_tasks:
-        task.cancel()
-
     msg = ""
     for record in records:
         _msg = str(record.message)
@@ -342,7 +345,7 @@ async def test_websocket_send(send, receive, message_stack):
 
 
 @pytest.mark.asyncio
-async def test_websocket_receive(send, receive, message_stack):
+async def test_websocket_text_receive(send, receive, message_stack):
     msg = {"text": "hello", "type": "websocket.receive"}
     message_stack.append(msg)
 
@@ -352,6 +355,17 @@ async def test_websocket_receive(send, receive, message_stack):
     assert text == msg["text"]
 
 
+@pytest.mark.asyncio
+async def test_websocket_bytes_receive(send, receive, message_stack):
+    msg = {"bytes": b"hello", "type": "websocket.receive"}
+    message_stack.append(msg)
+
+    ws = WebSocketConnection(send, receive)
+    data = await ws.receive()
+
+    assert data == msg["bytes"]
+
+
 @pytest.mark.asyncio
 async def test_websocket_accept_with_no_subprotocols(
     send, receive, message_stack
@@ -572,15 +586,28 @@ async def test_error_on_lifespan_exception_start(app, caplog):
     async def before_server_start(_):
         1 / 0
 
-    recv = AsyncMock(return_value={"type": "lifespan.startup"})
+    recv = AsyncMock(
+        side_effect=[
+            {"type": "lifespan.startup"},
+            {"type": "lifespan.shutdown"},
+        ]
+    )
     send = AsyncMock()
     app.asgi = True
 
+    lifespan = Lifespan(app, {"type": "lifespan"}, recv, send)
     with caplog.at_level(logging.ERROR):
-        await ASGIApp.create(app, {"type": "lifespan"}, recv, send)
+        await lifespan()
 
-    send.assert_awaited_once_with(
-        {"type": "lifespan.startup.failed", "message": "division by zero"}
+    send.assert_has_calls(
+        [
+            call(
+                {
+                    "type": "lifespan.startup.failed",
+                    "message": "division by zero",
+                }
+            )
+        ]
     )
 
 
@@ -590,13 +617,63 @@ async def test_error_on_lifespan_exception_stop(app: Sanic):
     async def before_server_stop(_):
         1 / 0
 
-    recv = AsyncMock(return_value={"type": "lifespan.shutdown"})
+    recv = AsyncMock(
+        side_effect=[
+            {"type": "lifespan.startup"},
+            {"type": "lifespan.shutdown"},
+        ]
+    )
     send = AsyncMock()
     app.asgi = True
     await app._startup()
 
-    await ASGIApp.create(app, {"type": "lifespan"}, recv, send)
+    lifespan = Lifespan(app, {"type": "lifespan"}, recv, send)
+    await lifespan()
 
-    send.assert_awaited_once_with(
-        {"type": "lifespan.shutdown.failed", "message": "division by zero"}
+    send.assert_has_calls(
+        [
+            call(
+                {
+                    "type": "lifespan.shutdown.failed",
+                    "message": "division by zero",
+                }
+            )
+        ]
     )
+
+
+@pytest.mark.asyncio
+async def test_asgi_headers_decoding(app: Sanic, monkeypatch: MonkeyPatch):
+    @app.get("/")
+    def handler(request: Request):
+        return text("")
+
+    headers_init = Headers.__init__
+
+    def mocked_headers_init(self, *args, **kwargs):
+        if "encoding" in kwargs:
+            kwargs.pop("encoding")
+        headers_init(self, encoding="utf-8", *args, **kwargs)
+
+    monkeypatch.setattr(Headers, "__init__", mocked_headers_init)
+
+    message = "Header names can only contain US-ASCII characters"
+    with pytest.raises(BadRequest, match=message):
+        _, response = await app.asgi_client.get("/", headers={"😂": "😅"})
+
+    _, response = await app.asgi_client.get("/", headers={"Test-Header": "😅"})
+    assert response.status_code == 200
+
+
+@pytest.mark.asyncio
+async def test_asgi_url_decoding(app):
+    @app.get("/dir/<name>", unquote=True)
+    def _request(request: Request, name):
+        return text(name)
+
+    # 2F should not become a path separator (unquoted later)
+    _, response = await app.asgi_client.get("/dir/some%2Fpath")
+    assert response.text == "some/path"
+
+    _, response = await app.asgi_client.get("/dir/some%F0%9F%98%80path")
+    assert response.text == "some😀path"
