@@ -303,6 +303,10 @@ def test_bp_with_host_list(app: Sanic):
 
     assert response.text == "Hello subdomain!"
 
+    route_names = [r.name for r in app.router.routes]
+    assert "test_bp_with_host_list.test_bp_host.handler1" in route_names
+    assert "test_bp_with_host_list.test_bp_host.handler2" in route_names
+
 
 def test_several_bp_with_host_list(app: Sanic):
     bp = Blueprint(
@@ -1088,8 +1092,7 @@ def test_remove_double_slashes_defined_on_bp(app: Sanic):
     bp = Blueprint("bp", url_prefix="/foo/", strict_slashes=True)
 
     @bp.get("/")
-    async def handler(_):
-        ...
+    async def handler(_): ...
 
     app.blueprint(bp)
     app.router.finalize()
@@ -1101,10 +1104,49 @@ def test_remove_double_slashes_defined_on_register(app: Sanic):
     bp = Blueprint("bp")
 
     @bp.get("/")
-    async def index(_):
-        ...
+    async def index(_): ...
 
     app.blueprint(bp, url_prefix="/foo/", strict_slashes=True)
     app.router.finalize()
 
     assert app.router.routes[0].path == "foo/"
+
+
+def test_blueprint_copy_returns_blueprint_with_the_name_of_original_blueprint(
+    app: Sanic,
+):
+    # arrange
+    bp = Blueprint("bp")
+
+    # act
+    actual = bp.copy("new_bp_name")
+
+    # assert
+    assert bp.name == actual.copied_from
+
+
+def test_blueprint_copy_returns_blueprint_with_overwritten_properties(
+    app: Sanic,
+):
+    # arrange
+    bp = Blueprint("bp")
+    to_override_attrs = expected = dict(
+        url_prefix="v2",
+        version="v2",
+        version_prefix="v2",
+        allow_route_overwrite=True,
+        strict_slashes=True,
+    )
+
+    # act
+    actual = bp.copy(
+        "new_bp_name",
+        **to_override_attrs,
+    )
+
+    # assert
+    assert all(
+        value == getattr(actual, key)
+        for key, value in expected.items()
+        if hasattr(actual, key)
+    )
