@@ -55,9 +55,8 @@ def test_create_task_with_app_arg(app):
 
 
 @pytest.mark.skipif(sys.version_info < (3, 8), reason="Not supported in 3.7")
-def test_create_named_task(app):
-    async def dummy():
-        ...
+def test_create_named_task(app, port):
+    async def dummy(): ...
 
     @app.before_server_start
     async def setup(app, _):
@@ -74,7 +73,7 @@ def test_create_named_task(app):
 
         app.stop()
 
-    app.run(single_process=True)
+    app.run(single_process=True, port=port)
 
 
 def test_named_task_called(app):
@@ -98,8 +97,7 @@ def test_named_task_called(app):
 
 @pytest.mark.skipif(sys.version_info < (3, 8), reason="Not supported in 3.7")
 def test_create_named_task_fails_outside_app(app):
-    async def dummy():
-        ...
+    async def dummy(): ...
 
     message = "Cannot name task outside of a running application"
     with pytest.raises(RuntimeError, match=message):
