@@ -7,7 +7,7 @@ from unittest.mock import Mock
 import pytest
 
 from bs4 import BeautifulSoup
-from pytest import LogCaptureFixture, MonkeyPatch, WarningsRecorder
+from pytest import LogCaptureFixture, MonkeyPatch
 
 from sanic import Sanic, handlers
 from sanic.exceptions import BadRequest, Forbidden, NotFound, ServerError
@@ -62,7 +62,6 @@ def exception_handler_app():
 
     @exception_handler_app.route("/8", error_format="html")
     def handler_8(request):
-
         raise ErrorWithRequestCtx("OK")
 
     @exception_handler_app.exception(ErrorWithRequestCtx, NotFound)
@@ -124,10 +123,10 @@ def test_html_traceback_output_in_debug_mode(exception_handler_app: Sanic):
     assert "handler_4" in html
     assert "foo = bar" in html
 
-    summary_text = " ".join(soup.select(".summary")[0].text.split())
-    assert (
-        "NameError: name 'bar' is not defined while handling path /4"
-    ) == summary_text
+    summary_text = soup.select("h3")[0].text
+    assert "NameError: name 'bar' is not defined" == summary_text
+    request_text = soup.select("h2")[-1].text
+    assert "GET /4" == request_text
 
 
 def test_inherited_exception_handler(exception_handler_app: Sanic):
@@ -147,11 +146,10 @@ def test_chained_exception_handler(exception_handler_app: Sanic):
     assert "handler_6" in html
     assert "foo = 1 / arg" in html
     assert "ValueError" in html
+    assert "GET /6" in html
 
-    summary_text = " ".join(soup.select(".summary")[0].text.split())
-    assert (
-        "ZeroDivisionError: division by zero while handling path /6/0"
-    ) == summary_text
+    summary_text = soup.select("h3")[0].text
+    assert "ZeroDivisionError: division by zero" == summary_text
 
 
 def test_exception_handler_lookup(exception_handler_app: Sanic):
@@ -171,7 +169,7 @@ def test_exception_handler_lookup(exception_handler_app: Sanic):
         pass
 
     try:
-        ModuleNotFoundError
+        ModuleNotFoundError  # noqa: F823
     except Exception:
 
         class ModuleNotFoundError(ImportError):
@@ -214,7 +212,7 @@ def test_error_handler_noisy_log(
     exception_handler_app: Sanic, monkeypatch: MonkeyPatch
 ):
     err_logger = Mock()
-    monkeypatch.setattr(handlers, "error_logger", err_logger)
+    monkeypatch.setattr(handlers.error, "error_logger", err_logger)
 
     exception_handler_app.config["NOISY_EXCEPTIONS"] = False
     exception_handler_app.test_client.get("/1")
@@ -268,20 +266,15 @@ def test_exception_handler_response_was_sent(
     assert "Error" in response.text
 
 
-def test_warn_on_duplicate(
-    app: Sanic, caplog: LogCaptureFixture, recwarn: WarningsRecorder
-):
+def test_errir_on_duplicate(app: Sanic):
     @app.exception(ServerError)
-    async def exception_handler_1(request, exception):
-        ...
+    async def exception_handler_1(request, exception): ...
 
-    @app.exception(ServerError)
-    async def exception_handler_2(request, exception):
-        ...
-
-    assert len(caplog.records) == 1
-    assert len(recwarn) == 1
-    assert caplog.records[0].message == (
+    message = (
         "Duplicate exception handler definition on: route=__ALL_ROUTES__ and "
         "exception=<class 'sanic.exceptions.ServerError'>"
     )
+    with pytest.raises(ServerError, match=message):
+
+        @app.exception(ServerError)
+        async def exception_handler_2(request, exception): ...
