@@ -16,8 +16,9 @@ from sanic_testing.testing import (
 )
 
 from sanic import Blueprint, Sanic
+from sanic.constants import DEFAULT_HTTP_CONTENT_TYPE
 from sanic.exceptions import ServerError
-from sanic.request import DEFAULT_HTTP_CONTENT_TYPE, RequestParameters
+from sanic.request import RequestParameters
 from sanic.response import html, json, text
 
 
@@ -104,11 +105,11 @@ def test_html(app):
         return html("<h1>Hello</h1>")
 
     @app.route("/foo")
-    async def handler(request):
+    async def handler_foo(request):
         return html(Foo())
 
     @app.route("/bar")
-    async def handler(request):
+    async def handler_bar(request):
         return html(Bar())
 
     request, response = app.test_client.get("/")
@@ -500,7 +501,8 @@ def test_standard_forwarded(app):
     headers = {
         "Forwarded": (
             'for=1.1.1.1, for=injected;host="'
-            ', for="[::2]";proto=https;host=me.tld;path="/app/";secret=mySecret'
+            ', for="[::2]";proto=https;host=me.tld;'
+            'path="/app/";secret=mySecret'
             ",for=broken;;secret=b0rked"
             ", for=127.0.0.3;scheme=http;port=1234"
         ),
@@ -512,6 +514,7 @@ def test_standard_forwarded(app):
     request, response = app.test_client.get("/", headers=headers)
     assert response.json == {"for": "127.0.0.2", "proto": "ws"}
     assert request.remote_addr == "127.0.0.2"
+    assert request.client_ip == "127.0.0.2"
     assert request.scheme == "ws"
     assert request.server_name == "local.site"
     assert request.server_port == 80
@@ -614,7 +617,8 @@ async def test_standard_forwarded_asgi(app):
     headers = {
         "Forwarded": (
             'for=1.1.1.1, for=injected;host="'
-            ', for="[::2]";proto=https;host=me.tld;path="/app/";secret=mySecret'
+            ', for="[::2]";proto=https;host=me.tld;'
+            'path="/app/";secret=mySecret'
             ",for=broken;;secret=b0rked"
             ", for=127.0.0.3;scheme=http;port=1234"
         ),
@@ -736,6 +740,7 @@ def test_remote_addr_with_two_proxies(app):
     headers = {"X-Forwarded-For": "127.0.1.1"}
     request, response = app.test_client.get("/", headers=headers)
     assert request.remote_addr == ""
+    assert request.client_ip == "127.0.0.1"
     assert response.body == b""
 
     headers = {"X-Forwarded-For": "127.0.0.1, 127.0.1.2"}
@@ -1247,7 +1252,8 @@ async def test_request_string_representation_asgi(app):
     [
         (
             "------sanic\r\n"
-            'Content-Disposition: form-data; filename="filename"; name="test"\r\n'
+            'Content-Disposition: form-data; filename="filename"'
+            '; name="test"\r\n'
             "\r\n"
             "OK\r\n"
             "------sanic--\r\n",
@@ -1255,7 +1261,8 @@ async def test_request_string_representation_asgi(app):
         ),
         (
             "------sanic\r\n"
-            'content-disposition: form-data; filename="filename"; name="test"\r\n'
+            'content-disposition: form-data; filename="filename"'
+            '; name="test"\r\n'
             "\r\n"
             'content-type: application/json; {"field": "value"}\r\n'
             "------sanic--\r\n",
@@ -1279,37 +1286,41 @@ async def test_request_string_representation_asgi(app):
         ),
         (
             "------sanic\r\n"
-            'Content-Disposition: form-data; filename*="utf-8\'\'filename_%C2%A0_test"; name="test"\r\n'
+            "Content-Disposition: form-data; filename*="
+            '"utf-8\'\'filename_%C2%A0_test"; name="test"\r\n'
             "\r\n"
             "OK\r\n"
             "------sanic--\r\n",
-            "filename_\u00A0_test",
+            "filename_\u00a0_test",
         ),
         (
             "------sanic\r\n"
-            'content-disposition: form-data; filename*="utf-8\'\'filename_%C2%A0_test"; name="test"\r\n'
+            "content-disposition: form-data; filename*="
+            '"utf-8\'\'filename_%C2%A0_test"; name="test"\r\n'
             "\r\n"
             'content-type: application/json; {"field": "value"}\r\n'
             "------sanic--\r\n",
-            "filename_\u00A0_test",
+            "filename_\u00a0_test",
         ),
         # Umlaut using NFC normalization (Windows, Linux, Android)
         (
             "------sanic\r\n"
-            'content-disposition: form-data; filename*="utf-8\'\'filename_%C3%A4_test"; name="test"\r\n'
+            "content-disposition: form-data; filename*="
+            '"utf-8\'\'filename_%C3%A4_test"; name="test"\r\n'
             "\r\n"
             "OK\r\n"
             "------sanic--\r\n",
-            "filename_\u00E4_test",
+            "filename_\u00e4_test",
         ),
         # Umlaut using NFD normalization (MacOS client)
         (
             "------sanic\r\n"
-            'content-disposition: form-data; filename*="utf-8\'\'filename_a%CC%88_test"; name="test"\r\n'
+            "content-disposition: form-data; filename*="
+            '"utf-8\'\'filename_a%CC%88_test"; name="test"\r\n'
             "\r\n"
             "OK\r\n"
             "------sanic--\r\n",
-            "filename_\u00E4_test",  # Sanic should normalize to NFC
+            "filename_\u00e4_test",  # Sanic should normalize to NFC
         ),
     ],
 )
@@ -1329,7 +1340,8 @@ def test_request_multipart_files(app, payload, filename):
     [
         (
             "------sanic\r\n"
-            'Content-Disposition: form-data; filename="filename"; name="test"\r\n'
+            'Content-Disposition: form-data; filename="filename";'
+            ' name="test"\r\n'
             "\r\n"
             "OK\r\n"
             "------sanic--\r\n",
@@ -1337,7 +1349,8 @@ def test_request_multipart_files(app, payload, filename):
         ),
         (
             "------sanic\r\n"
-            'content-disposition: form-data; filename="filename"; name="test"\r\n'
+            'content-disposition: form-data; filename="filename";'
+            ' name="test"\r\n'
             "\r\n"
             'content-type: application/json; {"field": "value"}\r\n'
             "------sanic--\r\n",
@@ -1361,19 +1374,21 @@ def test_request_multipart_files(app, payload, filename):
         ),
         (
             "------sanic\r\n"
-            'Content-Disposition: form-data; filename*="utf-8\'\'filename_%C2%A0_test"; name="test"\r\n'
+            "Content-Disposition: form-data; filename*="
+            '"utf-8\'\'filename_%C2%A0_test"; name="test"\r\n'
             "\r\n"
             "OK\r\n"
             "------sanic--\r\n",
-            "filename_\u00A0_test",
+            "filename_\u00a0_test",
         ),
         (
             "------sanic\r\n"
-            'content-disposition: form-data; filename*="utf-8\'\'filename_%C2%A0_test"; name="test"\r\n'
+            "content-disposition: form-data; filename*="
+            '"utf-8\'\'filename_%C2%A0_test"; name="test"\r\n'
             "\r\n"
             'content-type: application/json; {"field": "value"}\r\n'
             "------sanic--\r\n",
-            "filename_\u00A0_test",
+            "filename_\u00a0_test",
         ),
     ],
 )
@@ -1396,7 +1411,8 @@ def test_request_multipart_file_with_json_content_type(app):
 
     payload = (
         "------sanic\r\n"
-        'Content-Disposition: form-data; name="file"; filename="test.json"\r\n'
+        'Content-Disposition: form-data; name="file";'
+        ' filename="test.json"\r\n'
         "Content-Type: application/json\r\n"
         "Content-Length: 0"
         "\r\n"
@@ -1418,7 +1434,8 @@ async def test_request_multipart_file_with_json_content_type_asgi(app):
 
     payload = (
         "------sanic\r\n"
-        'Content-Disposition: form-data; name="file"; filename="test.json"\r\n'
+        'Content-Disposition: form-data; name="file";'
+        ' filename="test.json"\r\n'
         "Content-Type: application/json\r\n"
         "Content-Length: 0"
         "\r\n"
@@ -1480,7 +1497,8 @@ def test_request_multipart_file_duplicate_filed_name(app):
     )
 
     headers = {
-        "Content-Type": "multipart/form-data; boundary=e73ffaa8b1b2472b8ec848de833cb05b"
+        "Content-Type": "multipart/form-data;"
+        " boundary=e73ffaa8b1b2472b8ec848de833cb05b"
     }
 
     request, _ = app.test_client.post(
@@ -1515,7 +1533,8 @@ async def test_request_multipart_file_duplicate_filed_name_asgi(app):
     )
 
     headers = {
-        "Content-Type": "multipart/form-data; boundary=e73ffaa8b1b2472b8ec848de833cb05b"
+        "Content-Type": "multipart/form-data;"
+        " boundary=e73ffaa8b1b2472b8ec848de833cb05b"
     }
 
     request, _ = await app.asgi_client.post("/", data=payload, headers=headers)
@@ -1531,9 +1550,11 @@ def test_request_multipart_with_multiple_files_and_type(app):
         return text("OK")
 
     payload = (
-        '------sanic\r\nContent-Disposition: form-data; name="file"; filename="test.json"'
+        '------sanic\r\nContent-Disposition: form-data; name="file";'
+        ' filename="test.json"'
         "\r\nContent-Type: application/json\r\n\r\n\r\n"
-        '------sanic\r\nContent-Disposition: form-data; name="file"; filename="some_file.pdf"\r\n'
+        '------sanic\r\nContent-Disposition: form-data; name="file";'
+        ' filename="some_file.pdf"\r\n'
         "Content-Type: application/pdf\r\n\r\n\r\n------sanic--"
     )
     headers = {"content-type": "multipart/form-data; boundary=------sanic"}
@@ -1551,9 +1572,11 @@ async def test_request_multipart_with_multiple_files_and_type_asgi(app):
         return text("OK")
 
     payload = (
-        '------sanic\r\nContent-Disposition: form-data; name="file"; filename="test.json"'
+        '------sanic\r\nContent-Disposition: form-data; name="file";'
+        ' filename="test.json"'
         "\r\nContent-Type: application/json\r\n\r\n\r\n"
-        '------sanic\r\nContent-Disposition: form-data; name="file"; filename="some_file.pdf"\r\n'
+        '------sanic\r\nContent-Disposition: form-data; name="file";'
+        ' filename="some_file.pdf"\r\n'
         "Content-Type: application/pdf\r\n\r\n\r\n------sanic--"
     )
     headers = {"content-type": "multipart/form-data; boundary=------sanic"}
@@ -1813,8 +1836,8 @@ def test_request_cookies(app):
 
     request, response = app.test_client.get("/", cookies=cookies)
 
-    assert request.cookies == cookies
-    assert request.cookies == cookies  # For request._cookies
+    assert len(request.cookies) == len(cookies)
+    assert request.cookies["test"] == cookies["test"]
 
 
 @pytest.mark.asyncio
@@ -1827,8 +1850,8 @@ async def test_request_cookies_asgi(app):
 
     request, response = await app.asgi_client.get("/", cookies=cookies)
 
-    assert request.cookies == cookies
-    assert request.cookies == cookies  # For request._cookies
+    assert len(request.cookies) == len(cookies)
+    assert request.cookies["test"] == cookies["test"]
 
 
 def test_request_cookies_without_cookies(app):
@@ -2008,11 +2031,11 @@ def test_server_name_and_url_for(app):
     app.config.SERVER_NAME = "my-server"  # This means default port
     assert app.url_for("handler", _external=True) == "http://my-server/foo"
     request, response = app.test_client.get("/foo")
-    assert request.url_for("handler") == f"http://my-server/foo"
+    assert request.url_for("handler") == "http://my-server/foo"
 
     app.config.SERVER_NAME = "https://my-server/path"
     request, response = app.test_client.get("/foo")
-    url = f"https://my-server/path/foo"
+    url = "https://my-server/path/foo"
     assert app.url_for("handler", _external=True) == url
     assert request.url_for("handler") == url
 
@@ -2177,7 +2200,7 @@ def test_safe_method_with_body_ignored(app):
     )
 
     assert request.body == b""
-    assert request.json == None
+    assert request.json is None
     assert response.body == b"OK"
 
 
@@ -2198,10 +2221,27 @@ def test_safe_method_with_body(app):
     assert response.body == b"OK"
 
 
-def test_conflicting_body_methods_overload(app):
+@pytest.mark.asyncio
+async def test_conflicting_body_methods_overload_error(app: Sanic):
     @app.put("/")
     @app.put("/p/")
     @app.put("/p/<foo>")
+    async def put(request, foo=None): ...
+
+    with pytest.raises(
+        ServerError,
+        match=(
+            r"Duplicate route names detected:"
+            r" test_conflicting_body_methods_overload_error\.put.*"
+        ),
+    ):
+        await app._startup()
+
+
+def test_conflicting_body_methods_overload(app: Sanic):
+    @app.put("/", name="one")
+    @app.put("/p/", name="two")
+    @app.put("/p/<foo>", name="three")
     async def put(request, foo=None):
         return json(
             {"name": request.route.name, "body": str(request.body), "foo": foo}
@@ -2219,21 +2259,21 @@ def test_conflicting_body_methods_overload(app):
     _, response = app.test_client.put("/", json=payload)
     assert response.status == 200
     assert response.json == {
-        "name": "test_conflicting_body_methods_overload.put",
+        "name": "test_conflicting_body_methods_overload.one",
         "foo": None,
         "body": data,
     }
     _, response = app.test_client.put("/p", json=payload)
     assert response.status == 200
     assert response.json == {
-        "name": "test_conflicting_body_methods_overload.put",
+        "name": "test_conflicting_body_methods_overload.two",
         "foo": None,
         "body": data,
     }
     _, response = app.test_client.put("/p/test", json=payload)
     assert response.status == 200
     assert response.json == {
-        "name": "test_conflicting_body_methods_overload.put",
+        "name": "test_conflicting_body_methods_overload.three",
         "foo": "test",
         "body": data,
     }
@@ -2246,9 +2286,28 @@ def test_conflicting_body_methods_overload(app):
     }
 
 
-def test_handler_overload(app):
+@pytest.mark.asyncio
+async def test_handler_overload_error(app: Sanic):
     @app.get("/long/sub/route/param_a/<param_a:str>/param_b/<param_b:str>")
     @app.post("/long/sub/route/")
+    def handler(request, **kwargs): ...
+
+    with pytest.raises(
+        ServerError,
+        match=(
+            r"Duplicate route names detected:"
+            r" test_handler_overload_error\.handler.*"
+        ),
+    ):
+        await app._startup()
+
+
+def test_handler_overload(app: Sanic):
+    @app.get(
+        "/long/sub/route/param_a/<param_a:str>/param_b/<param_b:str>",
+        name="one",
+    )
+    @app.post("/long/sub/route/", name="two")
     def handler(request, **kwargs):
         return json(kwargs)
 
