@@ -20,9 +20,11 @@ from pytest import LogCaptureFixture
 
 from sanic import Request, Sanic
 from sanic.compat import Header
+from sanic.constants import DEFAULT_HTTP_CONTENT_TYPE
 from sanic.cookies import CookieJar
 from sanic.response import (
     HTTPResponse,
+    ResponseStream,
     empty,
     file,
     file_stream,
@@ -176,6 +178,10 @@ def json_app(app):
     async def unmodified_handler(request: Request):
         return json(JSON_DATA, status=304)
 
+    @app.get("/precondition")
+    async def precondition_handler(request: Request):
+        return json(JSON_DATA, status=412)
+
     @app.delete("/")
     async def delete_handler(request: Request):
         return json(None, status=204)
@@ -191,6 +197,10 @@ def test_json_response(json_app):
     assert response.text == json_dumps(JSON_DATA)
     assert response.json == JSON_DATA
 
+    request, response = json_app.test_client.get("/precondition")
+    assert response.status == 412
+    assert response.json == JSON_DATA
+
 
 def test_no_content(json_app):
     request, response = json_app.test_client.get("/no-content")
@@ -514,7 +524,6 @@ def test_file_stream_head_response(
 def test_file_stream_response_range(
     app: Sanic, file_name, static_file_directory, size, start, end
 ):
-
     Range = namedtuple("Range", ["size", "start", "end", "total"])
     total = len(get_file_content(static_file_directory, file_name))
     range = Range(size=size, start=start, end=end, total=total)
@@ -545,7 +554,7 @@ def test_raw_response(app):
         return raw(b"raw_response")
 
     request, response = app.test_client.get("/test")
-    assert response.content_type == "application/octet-stream"
+    assert response.content_type == DEFAULT_HTTP_CONTENT_TYPE
     assert response.body == b"raw_response"
 
 
@@ -574,14 +583,20 @@ def test_direct_response_stream(app: Sanic):
     assert "Content-Length" not in response.headers
 
 
-def test_two_respond_calls(app: Sanic):
+@pytest.mark.asyncio
+async def test_direct_response_stream_asgi(app: Sanic):
     @app.route("/")
-    async def handler(request: Request):
-        response = await request.respond()
+    async def test(request: Request):
+        response = await request.respond(content_type="text/csv")
         await response.send("foo,")
         await response.send("bar")
         await response.eof()
 
+    _, response = await app.asgi_client.get("/")
+    assert response.text == "foo,bar"
+    assert response.headers["Content-Type"] == "text/csv"
+    assert "Content-Length" not in response.headers
+
 
 def test_multiple_responses(
     app: Sanic,
@@ -609,7 +624,7 @@ def test_multiple_responses(
 
     @app.get("/4")
     async def handler4(request: Request):
-        response = await request.respond(headers={"one": "one"})
+        await request.respond(headers={"one": "one"})
         return json({"foo": "bar"}, headers={"one": "two"})
 
     @app.get("/5")
@@ -640,11 +655,6 @@ def test_multiple_responses(
         "been responded to."
     )
 
-    error_msg3 = (
-        "Response stream was ended, no more "
-        "response data is allowed to be sent."
-    )
-
     with caplog.at_level(ERROR):
         _, response = app.test_client.get("/1")
         assert response.status == 200
@@ -688,7 +698,7 @@ def test_multiple_responses(
         assert message_in_records(caplog.records, error_msg2)
 
 
-def send_response_after_eof_should_fail(
+def test_send_response_after_eof_should_fail(
     app: Sanic,
     caplog: LogCaptureFixture,
     message_in_records: Callable[[List[LogRecord], str], bool],
@@ -702,13 +712,14 @@ def send_response_after_eof_should_fail(
 
     error_msg1 = (
         "The error response will not be sent to the client for the following "
-        'exception:"Second respond call is not allowed.". A previous '
+        'exception:"Response stream was ended, no more response '
+        'data is allowed to be sent.". A previous '
         "response has at least partially been sent."
     )
 
     error_msg2 = (
-        "Response stream was ended, no more "
-        "response data is allowed to be sent."
+        "Response stream was ended, no more response "
+        "data is allowed to be sent."
     )
 
     with caplog.at_level(ERROR):
@@ -718,6 +729,39 @@ def send_response_after_eof_should_fail(
         assert message_in_records(caplog.records, error_msg2)
 
 
+@pytest.mark.asyncio
+async def test_send_response_after_eof_should_fail_asgi(
+    app: Sanic,
+    caplog: LogCaptureFixture,
+    message_in_records: Callable[[List[LogRecord], str], bool],
+):
+    @app.get("/")
+    async def handler(request: Request):
+        response = await request.respond()
+        await response.send("foo, ")
+        await response.eof()
+        await response.send("bar")
+
+    error_msg1 = (
+        "The error response will not be sent to the client for the "
+        'following exception:"There is no request to respond to, '
+        "either the response has already been sent or the request "
+        'has not been received yet.". A previous response has '
+        "at least partially been sent."
+    )
+
+    error_msg2 = (
+        "There is no request to respond to, either the response has "
+        "already been sent or the request has not been received yet."
+    )
+
+    with caplog.at_level(ERROR):
+        _, response = await app.asgi_client.get("/")
+        assert "foo, " in response.text
+        assert message_in_records(caplog.records, error_msg1)
+        assert message_in_records(caplog.records, error_msg2)
+
+
 @pytest.mark.parametrize(
     "file_name", ["test.file", "decode me.txt", "python.png"]
 )
@@ -768,7 +812,7 @@ def test_file_response_headers(
     assert (
         "cache-control" in headers
         and f"max-age={test_max_age}" in headers.get("cache-control")
-        and f"public" in headers.get("cache-control")
+        and "public" in headers.get("cache-control")
     )
     assert (
         "expires" in headers
@@ -799,14 +843,14 @@ def test_file_response_headers(
 
     _, response = app.test_client.get(f"/files/no_cache/{file_name}")
     headers = response.headers
-    assert "cache-control" in headers and f"no-cache" == headers.get(
+    assert "cache-control" in headers and "no-cache" == headers.get(
         "cache-control"
     )
     assert response.status == 200
 
     _, response = app.test_client.get(f"/files/no_store/{file_name}")
     headers = response.headers
-    assert "cache-control" in headers and f"no-store" == headers.get(
+    assert "cache-control" in headers and "no-store" == headers.get(
         "cache-control"
     )
     assert response.status == 200
@@ -894,7 +938,7 @@ def test_file_validating_invalid_header(
 @pytest.mark.parametrize(
     "file_name", ["test.file", "decode me.txt", "python.png"]
 )
-def test_file_validating_304_response(
+def test_file_validating_304_response_file_route(
     app: Sanic, file_name: str, static_file_directory: str
 ):
     @app.route("/files/<filename>", methods=["GET"])
@@ -944,3 +988,17 @@ def test_file_validating_304_response(
     )
     assert response.status == 304
     assert response.body == b""
+
+
+def test_stream_response_with_default_headers(app: Sanic):
+    async def sample_streaming_fn(response_):
+        await response_.write("foo")
+
+    @app.route("/")
+    async def test(request: Request):
+        return ResponseStream(sample_streaming_fn, content_type="text/csv")
+
+    _, response = app.test_client.get("/")
+    assert response.text == "foo"
+    assert response.headers["Transfer-Encoding"] == "chunked"
+    assert response.headers["Content-Type"] == "text/csv"
